require 'faster/erb'

require 'rake/clean'
CLEAN.include   'build/**/*.md'
CLOBBER.include 'build/*.pdf', 'build.pdf'

include Rake::DSL

ACTIONS = {}
ACTIONS[:build_pdf_with_pandoc] = lambda do |task|
  target = task.name
  sources = task.prerequisite_tasks.map(&:name)

  cmd = "pandoc --template=src/template.tex --chapters -o #{target} #{sources.join(' ')}"
  puts "Building #{target} with #{cmd.inspect}"

  system(cmd) || exit(1)
end

ACTIONS[:build_md_from_erb] = lambda do |task|
  target = task.name
  sources = task.prerequisite_tasks.map(&:name)
  puts "Building #{target}"

  File.open(target, 'w') do |f|
    sources.each do |source|
      f << Faster::ErbProcessor.new(source, File.read(source)).render
    end
  end
end

COMPILED_CHAPTERS = []
CHAPTERS.each do |chapter_name|

  chapter_file  = "src/chapters/#{chapter_name}.markdown.erb"
  result_md     = "build/chapters/#{chapter_name}.md"
  result_pdf    = "build/#{chapter_name}.pdf"

  file result_md  => chapter_file,  &ACTIONS[:build_md_from_erb]
  file result_pdf => result_md,     &ACTIONS[:build_pdf_with_pandoc]

  desc "Builds document with chapter #{chapter_name.inspect}"
  task "build:chapter:#{chapter_name}" => result_pdf

  COMPILED_CHAPTERS << result_md
end

file 'build.pdf' => COMPILED_CHAPTERS, &ACTIONS[:build_pdf_with_pandoc]
desc 'Builds document with all chapters'
task 'build' => 'build.pdf'


desc 'Generate basic directory structure'
task 'setup' do
  system 'mkdir -f build/chapters src/chapters'
end
